from cum.scrapers.foolslide import FoOlSlideChapter, FoOlSlideSeries
import re

BASE_URL = "http://www.yuri-ism.net/slide/"


class YuriismChapter(FoOlSlideChapter):
    BASE_URL = BASE_URL
    url_re = re.compile(r'http://www\.yuri-ism\.net/slide/read/')


class YuriismSeries(FoOlSlideSeries):
    BASE_URL = BASE_URL
    CHAPTER_OBJECT = YuriismChapter
    url_re = re.compile(r'http://www\.yuri-ism\.net/slide/series/')



from abc import ABCMeta, abstractmethod
from cum import config, exceptions
from cum.scrapers.base import BaseChapter, BaseSeries
from mimetypes import guess_extension
from tempfile import NamedTemporaryFile
from urllib.parse import urljoin, urlparse
import re
import requests


class FoOlSlideSeries(BaseSeries, metaclass=ABCMeta):
    def __init__(self, url, directory=None):
        self.url = url
        self.directory = directory
        self.get_comic_details()
        self.chapters = self.get_chapters()

    @property
    def api_hook_comic_details(self):
        path = 'api/reader/comic/id/{}'.format(self.foolslide_id)
        return urljoin(self.BASE_URL, path)

    @staticmethod
    def api_hook_comic_list(page):
        path = 'api/reader/comics/page/{}'.format(page)
        return urljoin(self.BASE_URL, path)

    @property
    @abstractmethod
    def BASE_URL(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def CHAPTER_OBJECT(self):
        raise NotImplementedError()

    def from_chapter_id(self):
        pass

    def get_comic_details(self):
        page = 1
        path = urlparse(self.url).path
        while True:
            response = requests.get(self.api_hook_comic_list(page)).json()
            if response.get('error', None) == 'Comics could not be found':
                break
            for comic in response['comics']:
                if urlparse(comic['href']).path == path:
                    self.foolslide_id = comic['id']
                    self.name = comic['name']
                    return
            page += 1

    def get_chapters(self):
        response = requests.get(self.api_hook_comic_details).json()
        chapters = []
        for chapter in response['chapters']:
            kwargs = {
                'name': self.name,
                'alias': self.alias,
                'chapter': chapter['chapter']['chapter'],
                'api_key': chapter['chapter']['id'],
                'url': chapter['chapter']['href'],
                'title': chapter['chapter']['name'],
                'groups': [team['name'] for team in chapter['teams']]
            }
            chapter = self.CHAPTER_OBJECT(**kwargs)
            chapters.append(chapter)
        return chapters

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value


class FoOlSlideChapter(BaseChapter, metaclass=ABCMeta):
    uses_pages = True
    chapter_id_re = re.compile(r'"chapter_id":"([0-9]*)"')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.api_key = kwargs.get('api_key')

    @staticmethod
    def api_hook_chapter_details(api_key):
        path = 'api/reader/chapter/id/{}'.format(api_key)
        return urljoin(self.BASE_URL, path)

    def download(self):
        response = (requests.get(self.api_hook_chapter_details(self.api_key))
                            .json())
        pages = response['pages']
        files = []
        with self.progress_bar(pages) as bar:
            for page in pages:
                r = requests.get(page['url'], stream=True)
                ext = guess_extension(r.headers.get('content-type'))
                f = NamedTemporaryFile(suffix=ext)
                for chunk in r.iter_content(chunk_size=4096):
                    if chunk:
                        f.write(chunk)
                f.flush()
                files.append(f)
                bar.update(1)
        self.create_zip(files)

    def from_url(url):
        r = requests.get(url)
        chapter_id = re.search(chapter_id_re, r.text).group(1)
        endpoint = FoOlSlideChapter.api_hook_chapter_details(chapter_id)
        response = requests.get(endpoint).json()
        response[]



import click
from math import ceil


def chapter(msg):
    click.echo(
        click.style('==> ', fg='green') + msg
    )


def configuration(dictionary):
    settings = configuration_flatten(dictionary)
    for setting, value in sorted(settings.items()):
        if value is not None:
            click.echo('{} = {}'.format(setting, value))


def configuration_flatten(dictionary, parent_key=None):
    items = []
    for key, value in dictionary.items():
        if parent_key:
            new_key = '{}.{}'.format(parent_key, key)
        else:
            new_key = key
        if isinstance(value, dict):
            items.extend(configuration_flatten(value, new_key).items())
        else:
            items.append((new_key, value))
    return dict(items)


def error(msg):
    click.echo(
        click.style('==> ', fg='red') + msg
    )


def list(items):
    """Print out a list of items in columns much like `ls` in bash."""
    if items:
        width = click.get_terminal_size()[0]
        longest = len(max(items, key=len))
        columns = int(width // (longest + 0.5))
        rows = ceil(len(items) / columns)

        for row in range(rows):
            line = []
            for column in range(columns):
                i = rows * column + row
                if i >= len(items):
                    continue
                line.append(items[i].ljust(longest))
            click.echo(' '.join(line))


def series(msg):
    click.echo(
        click.style('==> ', fg='blue', bold=True) + msg
    )


def warning(msg):
    click.echo(
        click.style('==> ', fg='yellow', bold=True) + msg
    )



from cum import db, config, output
from cum.scrapers import chapter_scrapers, series_scrapers
import click
import re


def chapter_by_url(url):
    """Helper function that iterates through the chapter scrapers defined in
    cum.scrapers.__init__ and returns an initialized chapter object when it
    matches the URL regex.
    """
    for Chapter in chapter_scrapers:
        if re.match(Chapter.url_re, url):
            return Chapter.from_url(url)


def list_new():
    """Helper method used in multiple cum commands to print out the new chapter
    details for each series. Has two possible styles for displaying the
    information: compact and normal.
    """
    items = {}
    for chapter in db.Chapter.find_new():
        try:
            items[chapter.alias].append(chapter.chapter)
        except KeyError:
            items[chapter.alias] = [chapter.chapter]
    if not items:
        return

    for series in sorted(items):
        if config.get().compact_new:
            print_new_compact(series, items)
        else:
            print_new_normal(series, items)


def print_new_compact(series, items):
    """Prints the new chapter information. E.g.
        joukamachi-no-dandelion 30  31  32  33  34
        minami-ke               153  154  155  156  157
    """
    longest_name = len(max(items, key=len))
    padding = longest_name - len(series)
    name = click.style(series + ' ' * padding, bold=True)
    chapters = '  '.join([x for x in items[series]])
    line = click.wrap_text(' '.join([name, chapters]),
                           subsequent_indent=' ' * (len(series) + padding + 1),
                           width=click.get_terminal_size()[0])
    click.echo(line)


def print_new_normal(series, items):
    """Prints the new chapter information. E.g.
        joukamachi-no-dandelion
        30  31  32  33  34
        minami-ke
        153  154  155  156  157
    """
    click.secho(series, bold=True)
    click.echo(click.wrap_text('  '.join([x for x in items[series]]),
                               width=click.get_terminal_size()[0]))


def series_by_url(url):
    """Helper function that iterates through the series scrapers defined in
    cum.scrapers.__init__ and returns an initialized series object when it
    matches the URL regex.
    """
    for Series in series_scrapers:
        if re.match(Series.url_re, url):
            return Series(url)


def set_ignored(mark_ignored, alias, chapters):
    """Helper function for `cum ignore` and `cum unignore` commands, which will
    either ignore chapters if mark_ignored is True or unignore chapters if
    mark_ignored is False.
    """
    if mark_ignored:
        downloaded = 0
        message_start = 'I'
        method = 'ignore'
    else:
        downloaded = -1
        message_start = 'Uni'
        method = 'mark_new'

    s = db.Series.alias_lookup(alias)
    query = (db.session.query(db.Chapter)
                       .filter(db.Chapter.series == s,
                               db.Chapter.downloaded == downloaded))
    if len(chapters) == 1 and chapters[0].lower() == 'all':
        click.echo('{}gnoring {} chapters for {}'
                   .format(message_start, len(s.chapters), s.name))
        click.confirm('Do you want to continue',
                      prompt_suffix='? ', abort=True)
    else:
        query = query.filter(db.Chapter.chapter.in_(chapters))

    chapters = [x.to_object() for x in query.all()]
    for chapter in chapters:
        function = getattr(chapter, method)
        function()
    if len(chapters) == 1:
        output.chapter('{}gnored chapter {} for {}'
                       .format(message_start, chapters[0].chapter, s.name))
    else:
        output.series('{}gnored {} chapters for {}'
                      .format(message_start, len(chapters), s.name))
